import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";
import { easyTasks, mediumTasks, hardTasks, dawnMessages, introMessages, moreTasksMessages } from "./taskData";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  const currentTime = world.getTimeOfDay();
  // Check if it's sunrise (0 ticks)
  if (currentTime === 20) {
    const players = world.getAllPlayers();
    players.forEach((player) => {
      player.sendMessage("§6" + dawnMessages[Math.floor(Math.random() * dawnMessages.length)]);
      assignNewTasks(player);
    });
  }

  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_tasks:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_tasks:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

function getRandomTask(tasks) {
  const randomIndex = Math.floor(Math.random() * tasks.length);
  return tasks[randomIndex];
}

function removeOldTaskTags(player) {
  const tags = player.getTags();
  tags.forEach((tag) => {
    if (tag.startsWith("sg_task:")) {
      player.removeTag(tag);
    }
  });
}

function assignNewTasks(player) {
  // Remove old task tags first
  removeOldTaskTags(player);

  // Assign one task from each difficulty level
  const easyTask = getRandomTask(easyTasks);
  const mediumTask = getRandomTask(mediumTasks);
  const hardTask = getRandomTask(hardTasks);

  player.addTag(`sg_task:${easyTask.taskID}`);
  player.addTag(`sg_task:${mediumTask.taskID}`);
  player.addTag(`sg_task:${hardTask.taskID}`);

  // You can display the tasks or store them in some way if needed
  // Example:
  player.sendMessage(`§aEasy:§r ${easyTask.description}`);
  player.sendMessage(`§eMedium:§r ${mediumTask.description}`);
  player.sendMessage(`§cHard:§r ${hardTask.description}`);
}


world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;
  const version = player.getDynamicProperty("sg_tasks_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_tasks_version", VERSION);
    if (player.hasTag("sg_tasks_received_guide")) {
      player.removeTag("sg_tasks_received_guide");
      player.sendMessage({
        translate: "sg_tasks.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_tasks:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_tasks_version", VERSION);
    player.sendMessage({
      translate: "sg_tasks.announce_update",
      with: [VERSION],
    });
  }
});



const BUTTONS = [
  {
    // OPTIONS
    translation_key: "sg_tasks.guide.tasks.title",
    icon_path: "textures/ui/creative_icon.png",
    action: (player) => {
      // Create and show the tasks form
      const form = new ActionFormData();
      form.title({
        translate: "sg_tasks.guide.tasks.title",
      });

      // Initialize an array to store the lines of the body
      let bodyLines = [];

      // Function to push a line to the bodyLines array
      const pushLine = (line) => {
        bodyLines.push(line);
      };

      // Function to check if the player has the required items
      const hasRequiredItems = (task) => {
        const inventoryComponent = player.getComponent(
          EntityComponentTypes.Inventory
        );
        if (!inventoryComponent) return false;

        const container = inventoryComponent.container;
        if (!container || typeof container.size !== "number") return false;

        // Create a map for item types and amounts in the inventory for faster lookups
        const inventoryItems = new Map();
        for (let i = 0; i < container.size; i++) {
          const itemStack = container.getItem(i);
          if (itemStack && itemStack.typeId && itemStack.amount) {
            if (inventoryItems.has(itemStack.typeId)) {
              inventoryItems.set(
                itemStack.typeId,
                inventoryItems.get(itemStack.typeId) + itemStack.amount
              );
            } else {
              inventoryItems.set(itemStack.typeId, itemStack.amount);
            }
          }
        }

        // Check if required items are present in the inventory
        for (const item of task.items) {
          const availableAmount = inventoryItems.get(item.typeID) || 0;
          if (availableAmount < item.amount) {
            return false;
          }
        }

        return true;
      };

      function collectRewards(player) {
        const tasks = [
          { difficulty: "EASY", taskArray: easyTasks },
          { difficulty: "MEDIUM", taskArray: mediumTasks },
          { difficulty: "HARD", taskArray: hardTasks },
        ];

        // Loop through each task difficulty level
        for (const { difficulty, taskArray } of tasks) {
          // Check which tasks the player has
          const playerTasks = taskArray.filter((task) =>
            player.hasTag(`sg_task:${task.taskID}`)
          );

          // If the player has any tasks, check for completion
          for (const task of playerTasks) {
            if (hasRequiredItems(task, player)) {
              // Call the function to offer rewards
              offerRewards(player, difficulty, task.taskID);
            }
          }
        }
      }

      function offerRewards(player, difficulty, taskID) {
        // Use loot tables to spawn items
        const lootTable = `${difficulty.toLowerCase()}`;
        const command = `/loot spawn ~ ~ ~ loot "sg_tasks/${lootTable}"`;

        player.runCommand(command);
        player.sendMessage(
          `§aRewards received for your ${difficulty.toLowerCase()} task!`
        );

        // Remove the task tag or mark it as completed
        removeTaskTag(player, taskID);
      }

      function removeTaskTag(player, taskID) {
        // Removes the task tag from the player
        player.removeTag(`sg_task:${taskID}`);
      }


      // Retrieve the player's task tags (assuming they are stored as tags like sg_task:1001)
      const taskTags = player
        .getTags()
        .filter((tag) => tag.startsWith("sg_task:"));

      if (taskTags.length === 0) {
        pushLine("You have completed all of your tasks for the day.\n\n§7Would you like to generate more?\n\n");
      } else {
        // Group tasks by difficulty level
        const easyTasksList = [];
        const mediumTasksList = [];
        const hardTasksList = [];

        for (const taskTag of taskTags) {
          // Extract the task ID from the tag (e.g., "sg_task:1001" -> 1001)
          const taskID = parseInt(taskTag.split(":")[1]);

          // Find the task based on the taskID and add to the corresponding array
          let task;
          if (taskID >= 1001 && taskID <= 1999) {
            task = easyTasks.find((t) => t.taskID === taskID);
            if (task) easyTasksList.push(task);
          } else if (taskID >= 2001 && taskID <= 2999) {
            task = mediumTasks.find((t) => t.taskID === taskID);
            if (task) mediumTasksList.push(task);
          } else if (taskID >= 3001 && taskID <= 3999) {
            task = hardTasks.find((t) => t.taskID === taskID);
            if (task) hardTasksList.push(task);
          }
        }
        const introText = introMessages(player);
        pushLine(
          introText[Math.floor(Math.random() * introText.length)] + "\n\n"
        );
        // Add each difficulty section to the body
        if (easyTasksList.length > 0) {
          easyTasksList.forEach((task) => {
            const taskStatus = hasRequiredItems(task)
              ? "§bREWARD AVAILABLE"
              : "§8INCOMPLETE";
            pushLine(`§aEASY - ${taskStatus}\n§f${task.description}\n\n`);
          });
        }

        if (mediumTasksList.length > 0) {
          mediumTasksList.forEach((task) => {
            const taskStatus = hasRequiredItems(task)
              ? "§bREWARD AVAILABLE"
              : "§8INCOMPLETE";
            pushLine(`§eMEDIUM - ${taskStatus}\n§f${task.description}\n\n`);
          });
        }

        if (hardTasksList.length > 0) {
          hardTasksList.forEach((task) => {
            const taskStatus = hasRequiredItems(task)
              ? "§bREWARD AVAILABLE"
              : "§8INCOMPLETE";
            pushLine(`§cHARD - ${taskStatus}\n§f${task.description}\n\n`);
          });
        }
        // Add the footer message
        pushLine(
          "§7Complete all tasks to get new ones,\nor wait/sleep until the next dawn."
        );
      }


      // Set the form body
      form.body(bodyLines.join(""));

      // Add a button based on whether any tasks have rewards available
      const hasRewardsAvailable = taskTags.some((taskTag) => {
        const taskID = parseInt(taskTag.split(":")[1]);
        const task = easyTasks
          .concat(mediumTasks, hardTasks)
          .find((t) => t.taskID === taskID);
        return task && hasRequiredItems(task);
      });
      if (taskTags.length === 0) {
        form.button("Generate New Tasks", "textures/ui/icon_random.png");
      } else {
        form.button(
          hasRewardsAvailable ? "Collect Rewards" : "No Rewards Available",
          hasRewardsAvailable ? "textures/ui/icon_blackfriday.png" : "textures/ui/lock_color.png"
        );
      }

      // Show the form to the player
      form.show(player).then((res) => {
        // Handle the button click
        if (taskTags.length === 0) {
          player.sendMessage("§6" + moreTasksMessages[Math.floor(Math.random() * moreTasksMessages.length)]);
          assignNewTasks(player);
        } else {
          if (res.selection === 0 && hasRewardsAvailable) {
            // Logic for collecting rewards goes here
            collectRewards(player);
          } else if (res.selection === 0 && !hasRewardsAvailable) {
            player.sendMessage(
              "§cNo rewards available, complete tasks to get rewards!"
            );
          }
        }
      });
    },
  },

  {
    // CHANGELOG
    translation_key: "sg_tasks.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_tasks.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_tasks.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_tasks.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_tasks.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_tasks.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_tasks.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_tasks:guide") {
      showMainPage(event.source);
    }
  });
});